==> hermes-5.1.0.11.rec2.tar.gz: hermes-5.1.0.11.rec2/app/models/hermes/concerns/sopra_xml_request_getter.rb <==
module Hermes
  module Concerns
    module SopraXmlRequestGetter
      extend ActiveSupport::Concern

      included do

        belongs_to(
          :xml,
          class_name: 'Hermes::Xml',
          index: true
        )

        def parse_xml(xml_string)
          Hash.from_xml(xml_string.gsub("\n", ""))
        end

        def create_xml_model(xml, user = nil)
          xml_model = Hermes::Xml.create(xml: xml, user: user, object_id: self.id, reference_model_name: self.class.name)
          set(updated_at: DateTime.now, xml_id: xml_model.id)
          xml_model.xml
        end 

        def get_xml_from_sne(user, unique_number = nil)
          client = Hermes::BoClient::Client.new
          so = Hermes::BoClient::FindNudTask.new(housing_request_initializer_id: id, unique_number: unique_number)
          begin
            unless Rails.env.test?
              endpoint = client.endpoint_for(so.endpoint)
              res = endpoint.post(so.to_xml, {})
              create_xml_model(res.body, user)
            else
              path = [Hermes::Engine.root, %w[spec xml_bouchon v4],   unique_number + ".XML"].flatten.join('/').to_s
              pn = Pathname.new(path)
              if(! pn.exist?)
                path = [Hermes::Engine.root, %w[spec xml_bouchon v4],   unique_number + ".xml"].flatten.join('/').to_s
                pn = Pathname.new(path)
              end
              return {errors: 'missing test file'} unless pn.exist?
              create_xml_model(File.read(pn), user)
            end 
          rescue => e 
            client.log_error("GetXMLError ----> "+e.inspect, so.endpoint, so.to_xml, self.class.name, id)
            Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_POST_ERROR", action: "#{self.class.name} : #{id} -- GetXMLError", data: { response: e.inspect }.as_json)
            extract_sopra_errors(e)
          end 
        end

        def extract_sopra_errors(res)
          begin
            error_hash = parse_xml(res.response.to_s.force_encoding('UTF-8'))
            errors = 
            if error_hash['integration_demande_reponse']
              if error_hash['integration_demande_reponse']['erreurs']['erreur'].is_a? Hash
                [error_hash['integration_demande_reponse']['erreurs']['erreur']['message']]
              else
                error_hash['integration_demande_reponse']['erreurs']['erreur'].map { |a| a['message']}
              end 
            elsif error_hash['errors']
              if error_hash['errors'].is_a? Hash
                [error_hash["errors"]['error']['message']['content']] 
              else
                error_hash["errors"]['error']['message']['content'] .map { |a| a['error']['message']['content'] }
              end
            else
              nil
            end 
        
            { errors: { sopra_error: errors }}
          rescue => e
            { errors: ["undefined sopra error"]}
          end
        end 

        def extract_email_from_xml
          return unless xml&.xml
          parsed_xml = parse_xml(xml.xml)
          parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["demandeur"]["mel"]
        end 
      

        def is_same_applicant(coapplicant, coap_params)
          to_check_params = ["first_name", "last_name", "birth_date"]
          score=0
          to_check_params.each do |param|
            if (coap_params[param.to_sym] == coapplicant.send(param))
              score += 1
            end
          end
          identity_score = 3 
          return score == identity_score
        end

        def delete_unnecessary_applicants(new_coaps, old_coaps)
          Hermes::Applicant.where(:id.in => (old_coaps.map{|c| c[:id] }.compact - new_coaps.map{|c| c[:id] }.compact)).delete_all
        end

        def compute_applicant_param(applicant_param, applicant)
        if(applicant.main_employer_name)
          applicant_param[:main_employer_name] = applicant.main_employer_name
        end
        applicant_param
        end

        def create_or_update_applicant(coapp_params, housing_request, is_colloc= false)
          coapplicants = Hermes::Applicant.where(:id.in => housing_request.coapplicants.map{|c| c[:id]}.compact)
          found=false
          created_applicant = nil
          coapplicants.each do |coapplicant|
              if(!found && is_same_applicant(coapplicant, coapp_params))
                found=true
                if !is_colloc
                  coapp_params = reset_applicant_workplace_locality_and_district(coapp_params, coapplicant)
                  coapplicant.update(compute_applicant_param(coapp_params, coapplicant))
                else
                  coapplicant.user_id = user_id || housing_request.user_id
                  coapplicant.save
                end
                created_applicant = coapplicant
              end
          end
          if( !found)
            created_applicant = Hermes::Applicant.create(coapp_params)
          end
          return created_applicant
        end

        def reset_applicant_workplace_locality_and_district(applicant_param, applicant)
          if applicant_param[:workplace_postal_code] != applicant.workplace_postal_code || 
            applicant_param[:workplace_insee_code] != applicant.workplace_insee_code 
            applicant_param[:workplace_locality] = nil
          else
            applicant_param[:workplace_locality] = applicant.workplace_locality 
          end 

          ["owned_housing_details", "actual_housing_details", "postal_details"].each do |address_detail|
            if applicant_param[address_detail.to_sym]&.fetch("address_insee_code", nil) != applicant.send(address_detail)&.fetch("address_insee_code", nil) 
              applicant_param[address_detail.to_sym][:district] = nil if applicant_param[address_detail.to_sym]
            else
              applicant_param[address_detail.to_sym][:district] =  applicant.send(address_detail)&.fetch("district", nil) if applicant_param[address_detail.to_sym]
            end 
          end
          applicant_param
        end 

        def manage_seeked_location_name(housing_request_params, housing_request)
          housing_request.seeked_localisation_list&.each_with_index do |hr_location, i|
            if hr_location&.fetch("nom", false) && hr_location&.fetch("code", nil) == housing_request_params[:seeked_localisation_list]&.fetch(i,{})&.fetch("code",nil)
              housing_request_params[:seeked_localisation_list][i]["nom"] = hr_location["nom"]
            end
          end
          housing_request_params
        end

        def get_new_housing_channel_keys(housing_request, extracted_request_attributes)
          new_keys = housing_request.housing_channel_keys
          new_keys = extracted_request_attributes[:dalo] ? new_keys + ['dalo'] : new_keys - ['dalo']
          new_keys.uniq
        end 

        def update_request_and_applicant_from_xml(parsed_xml, housing_request, main_applicant)
          if housing_request.coapplicants.any? {|c| c.blank?}
            housing_request.clean_coapplicants
            housing_request.reload
          end 
          main_applicant_param = Hermes::BoClient::Mappings::ApplicantReverseMap.new(parsed_xml, true).build_applicant
          main_applicant.update(reset_applicant_workplace_locality_and_district(compute_applicant_param(main_applicant_param, main_applicant), main_applicant))

          coapplicants_to_add = []
          if parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["listeCodemandeur"]
            xml_coapps = parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["listeCodemandeur"]["codemandeur"] 
            xml_coapps = xml_coapps.is_a?(Array) ? xml_coapps : [xml_coapps]
            xml_coapps.each_with_index do |xml_coapp, i|
              coapp_params = Hermes::BoClient::Mappings::ApplicantReverseMap.new(parsed_xml, false, i).build_applicant.merge(user_id: user_id || housing_request.user_id)
              if coapplicant = create_or_update_applicant(coapp_params, housing_request)
                coapplicants_to_add << { rel_type: lienDemandeur_v4_map(xml_coapps[i]["lienDemandeur"]["code"]),  id: coapplicant.id.to_s }
              end
            end
          end 
          #10113
          if parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["listeColocataire"]
            xml_colocs = parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["listeColocataire"]["colocataire"] 
            xml_colocs = xml_colocs.is_a?(Array) ? xml_colocs : [xml_colocs]
            xml_colocs.each_with_index do |xml_coloc, j|
              coloc_params = Hermes::BoClient::Mappings::ApplicantReverseMap.new(parsed_xml, false, j, true).build_applicant.merge(user_id: user_id || housing_request.user_id)
              if coapplicant = create_or_update_applicant(coloc_params,housing_request, true)
                coapplicants_to_add << { rel_type: "L",  id: coapplicant.id.to_s }
              end
            end
          end 
          delete_unnecessary_applicants(coapplicants_to_add,housing_request.coapplicants)
          extracted_request_attributes = manage_seeked_location_name(Hermes::BoClient::Mappings::HousingRequestReverseMap.new(parsed_xml).build_housing_request, housing_request)
          housing_request.update(
            extracted_request_attributes.merge(
              is_active: true,
              form_status: "invalid",
              owned_housing_details: main_applicant.owned_housing_details,
              actual_housing_details: main_applicant.actual_housing_details,
              postal_details: main_applicant.postal_details,
              coapplicants: coapplicants_to_add,
              viewed_by_colaborator_offre_logement: false,
              housing_channel_keys: get_new_housing_channel_keys(housing_request, extracted_request_attributes)
            )
          )
        end 
        def lienDemandeur_v4_map(code)
          map={
            "R"=>"M",
          }
          return map[code] || code
        end
      end
    end
  end
end

==> hermes-5.4.0.0.rec2.tar.gz: hermes-5.4.0.0.rec2/app/models/hermes/concerns/sopra_xml_request_getter.rb <==
module Hermes
  module Concerns
    module SopraXmlRequestGetter
      extend ActiveSupport::Concern

      included do

        belongs_to(
          :xml,
          class_name: 'Hermes::Xml',
          index: true
        )

        def parse_xml(xml_string)
          Hash.from_xml(xml_string.gsub("\n", ""))
        end

        def get_xmls(type = 'get_demande')
          Hermes::Xml.where(type: type, object_id: self.id, reference_model_name: self.class.name).order_by(created_at: :asc)
        end 

        def last_xml(type = 'get_demande')
          get_xmls(type)&.last&.xml
        end 

        def create_xml_model(xml, type, user = nil)
          xml_model = Hermes::Xml.create(xml: xml, user: user, type: type, object_id: self.id, reference_model_name: self.class.name)
          set(updated_at: DateTime.now, xml_id: xml_model.id)
          xml_model.xml
        end 

        def get_xml_from_sopra(opts={})
          client = Hermes::BoClient::Client.new
          so = opts[:so]
          user = opts[:user]
          unique_number = opts[:unique_number]
          type = opts[:type]

          begin
            if Rails.env.test?
              path =  case type
              when 'get_demande'
                [Hermes::Engine.root, %w[spec xml_bouchon v4], unique_number + ".xml"]
              when 'get_supplement'
                [Hermes::Engine.root, %w[spec xml_bouchon get_supplement], unique_number + ".xml"]
              end
              pn = Pathname.new(path.flatten.join('/').to_s)
              return {errors: 'missing test file'} unless pn.exist?
              create_xml_model(File.read(pn), type, user)
            else
              endpoint = client.endpoint_for(so.endpoint)
              res = endpoint.post(so.to_xml, {})
              create_xml_model(res.body, type, user)
            end 
          rescue => e 
            client.log_error("GetXMLError ----> "+e.inspect, so.endpoint, so.to_xml, self.class.name, id)
            Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_POST_ERROR", action: "#{self.class.name} : #{id} -- GetXMLError", data: { response: e.inspect }.as_json)
            extract_sopra_errors(e)
          end 
        end

        def get_xml_from_sne(user, unique_number = nil)
          so = Hermes::BoClient::FindNudTask.new(housing_request_initializer_id: id, unique_number: unique_number)
          get_xml_from_sopra(so:so, type:'get_demande', user:user, unique_number: unique_number)
        end

        def get_supplement_xml(user, unique_number = nil)
          so = Hermes::BoClient::SupplementRequestTask.new(housing_request_id: id)
          get_xml_from_sopra(so:so, type:'get_supplement', user: user, unique_number: unique_number)
        end

        def extract_sopra_errors(res)
          begin
            error_hash = parse_xml(res.response.to_s.force_encoding('UTF-8'))
            errors = 
            if error_hash['integration_demande_reponse']
              if error_hash['integration_demande_reponse']['erreurs']['erreur'].is_a? Hash
                [error_hash['integration_demande_reponse']['erreurs']['erreur']['message']]
              else
                error_hash['integration_demande_reponse']['erreurs']['erreur'].map { |a| a['message']}
              end 
            elsif error_hash['errors']
              if error_hash['errors'].is_a? Hash
                [error_hash["errors"]['error']['message']['content']] 
              else
                error_hash["errors"]['error']['message']['content'] .map { |a| a['error']['message']['content'] }
              end
            else
              nil
            end 
        
            { errors: { sopra_error: errors }}
          rescue => e
            { errors: ["undefined sopra error"]}
          end
        end 

        def extract_email_from_xml
          last = last_xml
          return unless last
          parsed_xml = parse_xml(last)
          parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["demandeur"]["mel"]
        end 

        def is_same_applicant(coapplicant, coap_params)
          to_check_params = ["first_name", "last_name", "birth_date"]
          score=0
          to_check_params.each do |param|
            if (coap_params[param.to_sym] == coapplicant.send(param))
              score += 1
            end
          end
          identity_score = 3 
          return score == identity_score
        end

        def delete_unnecessary_applicants(new_coaps, old_coaps)
          Hermes::Applicant.where(:id.in => (old_coaps.map{|c| c[:id] }.compact - new_coaps.map{|c| c[:id] }.compact)).delete_all
        end

        def compute_applicant_param(applicant_param, applicant)
          if(applicant.main_employer_name)
            applicant_param[:main_employer_name] = applicant.main_employer_name
          end
          applicant_param
        end

        def create_or_update_applicant(coapp_params, housing_request, is_colloc= false)
          coapplicants = Hermes::Applicant.where(:id.in => housing_request.coapplicants.map{|c| c[:id]}.compact)
          found=false
          created_applicant = nil
          coapplicants.each do |coapplicant|
              if(!found && is_same_applicant(coapplicant, coapp_params))
                found=true
                if !is_colloc
                  coapp_params = reset_applicant_workplace_locality_and_district(coapp_params, coapplicant)
                  coapplicant.update(compute_applicant_param(coapp_params, coapplicant))
                else
                  coapplicant.user_id = user_id || housing_request.user_id
                  coapplicant.save
                end
                created_applicant = coapplicant
              end
          end
          if( !found)
            created_applicant = Hermes::Applicant.create(coapp_params)
          end
          return created_applicant
        end

        def reset_applicant_workplace_locality_and_district(applicant_param, applicant)
          if applicant_param[:workplace_postal_code] != applicant.workplace_postal_code || 
            applicant_param[:workplace_insee_code] != applicant.workplace_insee_code 
            applicant_param[:workplace_locality] = nil
          else
            applicant_param[:workplace_locality] = applicant.workplace_locality 
          end 

          ["owned_housing_details", "actual_housing_details", "postal_details"].each do |address_detail|
            if applicant_param[address_detail.to_sym]&.fetch("address_insee_code", nil) != applicant.send(address_detail)&.fetch("address_insee_code", nil) 
              applicant_param[address_detail.to_sym][:district] = nil if applicant_param[address_detail.to_sym]
            else
              applicant_param[address_detail.to_sym][:district] =  applicant.send(address_detail)&.fetch("district", nil) if applicant_param[address_detail.to_sym]
            end 
          end
          applicant_param
        end 

        def manage_seeked_location_name(housing_request_params, housing_request)
          housing_request.seeked_localisation_list&.each_with_index do |hr_location, i|
            if hr_location&.fetch("nom", false) && hr_location&.fetch("code", nil) == housing_request_params[:seeked_localisation_list]&.fetch(i,{})&.fetch("code",nil)
              housing_request_params[:seeked_localisation_list][i]["nom"] = hr_location["nom"]
            end
          end
          housing_request_params
        end

        def get_new_housing_channel_keys(housing_request, extracted_request_attributes)
          new_keys = housing_request.housing_channel_keys
          new_keys = extracted_request_attributes[:dalo] ? new_keys + ['dalo'] : new_keys - ['dalo']
          new_keys.uniq
        end 

        def update_request_and_applicant_from_xml(parsed_xml, housing_request, user = nil)
          if housing_request.coapplicants.any? {|c| c.blank?}
            housing_request.clean_coapplicants
            housing_request.reload
          end 
          main_applicant = housing_request.main_applicant
          main_applicant_param = Hermes::BoClient::Mappings::ApplicantReverseMap.new(parsed_xml, true).build_applicant
          main_applicant.update(reset_applicant_workplace_locality_and_district(compute_applicant_param(main_applicant_param, main_applicant), main_applicant))

          coapplicants_to_add = []
          if parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["listeCodemandeur"]
            xml_coapps = parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["listeCodemandeur"]["codemandeur"] 
            xml_coapps = xml_coapps.is_a?(Array) ? xml_coapps : [xml_coapps]
            xml_coapps.each_with_index do |xml_coapp, i|
              coapp_params = Hermes::BoClient::Mappings::ApplicantReverseMap.new(parsed_xml, false, i).build_applicant.merge(user_id: user_id || housing_request.user_id)
              if coapplicant = create_or_update_applicant(coapp_params, housing_request)
                coapplicants_to_add << { rel_type: lienDemandeur_v4_map(xml_coapps[i]["lienDemandeur"]["code"]),  id: coapplicant.id.to_s }
              end
            end
          end 
          #10113
          if parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["listeColocataire"]
            xml_colocs = parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["listeColocataire"]["colocataire"] 
            xml_colocs = xml_colocs.is_a?(Array) ? xml_colocs : [xml_colocs]
            xml_colocs.each_with_index do |xml_coloc, j|
              coloc_params = Hermes::BoClient::Mappings::ApplicantReverseMap.new(parsed_xml, false, j, true).build_applicant.merge(user_id: user_id || housing_request.user_id)
              if coapplicant = create_or_update_applicant(coloc_params,housing_request, true)
                coapplicants_to_add << { rel_type: "L",  id: coapplicant.id.to_s }
              end
            end
          end 
          delete_unnecessary_applicants(coapplicants_to_add,housing_request.coapplicants)
          extracted_request_attributes = manage_seeked_location_name(Hermes::BoClient::Mappings::HousingRequestReverseMap.new(parsed_xml).build_housing_request, housing_request)
          housing_request.update(
            extracted_request_attributes.merge(
              is_active: true,
              form_status: "invalid",
              owned_housing_details: main_applicant.owned_housing_details,
              actual_housing_details: main_applicant.actual_housing_details,
              postal_details: main_applicant.postal_details,
              coapplicants: coapplicants_to_add,
              viewed_by_colaborator_offre_logement: false,
              housing_channel_keys: get_new_housing_channel_keys(housing_request, extracted_request_attributes)
            )
          )
          housing_request.update_request_supplement(user)
        end 

        def update_request_renewal_values_from_xml(parsed_xml, housing_request, user = nil)
          extracted_request_attributes = Hermes::BoClient::Mappings::HousingRequestReverseMap.new(parsed_xml).build_housing_request

          housing_request.update(extracted_request_attributes.slice(
            :sne_created_at, 
            :last_sne_renewal_date,
            :is_striked_off,
            :striked_off_motives,
          ))
        end 

        def update_request_supplement_values_from_xml(parsed_xml, housing_request, user = nil)
          extracted_request_attributes = Hermes::BoClient::Mappings::HousingRequestSupplementMap.new(parsed_xml).build_housing_request

          housing_request.update(extracted_request_attributes)
        end 

        def lienDemandeur_v4_map(code)
          map={
            "R"=>"M",
          }
          return map[code] || code
        end
      end
    end
  end
end
